/**
 * @mainpage
 *
 * @section INFORMÁCIE
 * @author  Patrik Slučiak | Diallix (c)
 * @date    08.5 2013
 * @version 9.2
 *
 * @section POPIS
 *
 * Tento projekt bol vytvorený ako pomocný modul pre konvertovanie medzi podporovanými údajovými typmi v C++.
 *
 * Vývojové prostredie: Microsoft Visual Studio 2010
 *
 * @section LICENCIA
 *
 * Modul je uverejnený bez komerčnej politiky ako free-ware a je duševným vlastníctvom autora __Patrik Slučiak | Diallix (c)__, ďalej len Autor.
 * Je výslovne zakázané privlastnovanie si a propagovanie pod vlastným menom tento modul, vrátanie komerčného šírenia a propagovania bez ďalšieho súhlasu Autora.
 * 
 * Návod a príkladové použitie modulu "Convert.h" je k dispozícií : https://www.netbot.sk/sk/14-blog-headers/84-convert-h
 * Pripomienky / Návrhy, prosím, píšte na diallix@diallix.net .
 * 
 * 
 * 
 */

/**
 * @file Convert.h
 * @brief Zdrojový súbor Convert.h
 *
 * 8.5.2013 - Prvná verzia
 *
 * 19.10.2020 - Posledná verzia
 */

#include <sstream>
#include <tchar.h>
#include <atlbase.h>

#ifndef _INC_STDLIB
	#include <stdlib.h>
#endif
#ifndef _NSTREAM_
	#include <stdio.h>
#endif

/**
 * @def Definovanie identifikátorov knižnice "Convert.h" .
 */
//Definovanie identifikátorov knižnice Convert.h//
#ifndef CP_ACP
	#define CP_ACP 0
#endif

#define _CVRT_ 195217
#define _Diall_CVRT _CVRT_

#define __us_size__ 10000
#define _LP_ LPCWSTR

using namespace std;

/*! \brief Trieda "Diall_Convert" pre konvertovanie knižnice "Convert.h" .
 *  
 *
   \brief Trieda implementujúca metódy pre konvertovanie dátových typov.
 * Jednotlivé podporované dátové typy:
 *
 *    STRING -> LPCWSTR
 *    
 *    CONST STRING -> WSTRING
 *
 *    LPTSTR -> STRING
 *
 * 	  STRING -> WSTRING
 *
 * 	  STRING -> LPVOID
 *
 * 	  WSTRING -> STRING
 *
 * 	  STRING -> INTEGER
 *
 * 	  INTEGER -> STRING
 *
 * 	  FLOAT -> STRING
 *
 * 	  STRING -> FLOAT
 *
 * 	  CHAR -> STRING
 * 
 *    CHAR -> INTEGER
 *
 */
class Diall_Convert {
private:
	  wchar_t * convertstring;
	  const char * x_string;
	  char * wsbuffer;
	  ::std::string svt_tmp;
	  ::std::stringstream dstreambuffer;
	  int cnvt_tmp;
	  int cnvt_l;
	  float dbuffer;
	  static Diall_Convert * instance;

public:
	static Diall_Convert * GetInstance(void)
	{
		if(!instance)
		{
			instance = new Diall_Convert();
		}
		return instance;
	}

private:
	 Diall_Convert(void) 
	 {
		 this->convertstring = NULL;
		 this->wsbuffer = NULL;
		 this->x_string = NULL;
		 this->ClearBuffer();
     };

public:

/**
 * @fn ::LPCWSTR String_To_LPCWSTR(::std::string convert_string)
 *
 * @brief Funkcia konvertovania "STRING" -> "LPCWSTR" (Long Pointer to Constant Wide String).
 *
 * @param convert_string Vstupná hodnota dátového typu "STRING".
 *
 * @return Funkcia vracia vstupnú hodnotu pretypovanú na "LPCWSTR".
 */
    ::LPCWSTR String_To_LPCWSTR(::std::string convert_string) 
	{
         const char * x_stringd = convert_string.c_str();
		 wchar_t * convertstringd = new wchar_t[__us_size__];
         ::mbstowcs(convertstringd,x_stringd, __us_size__);
		 return convertstringd;
	};

    void String_To_LPCWSTR_delete(void) 
	{
	     delete []convertstring;

	};

/**
 * @fn ::std::wstring s2ws(const ::std::string& s)
 *
 * @brief Funkcia konvertovania "CONST STRING -> WSTRING".
 *
 * @param convert_string Vstupná hodnota dátového typu "STRING".
 *
 * @return Funkcia vracia vstupnú hodnotu pretypovanú na "LPCWSTR".
 */
	::std::wstring s2ws(const ::std::string& convert_string)
	{
		int len;
		int slength = (int)convert_string.length() + 1;
		len = ::MultiByteToWideChar(CP_ACP, 0, convert_string.c_str(), slength, 0, 0); 
		wchar_t * buf = new wchar_t[len];
		::MultiByteToWideChar(CP_ACP, 0, convert_string.c_str(), slength, buf, len);
		::std::wstring output(buf);

		delete[]buf;

		return output;
	 }

/**
 * @fn ::std::string LPTSTR_To_String(::LPTSTR convert_LPTSTR) 
 *
 * @brief Funkcia konvertovania "LPTSTR" (non-const Tchar String) -> "STRING".
 *
 * @param convert_string Vstupná hodnota dátového typu "LPTSTR".
 *
 * @return Funkcia vracia vstupnú hodnotu pretypovanú na "STRING".
 */
	::std::string LPTSTR_To_String(::LPTSTR convert_LPTSTR) 
	{
         USES_CONVERSION;
         this->svt_tmp = W2A(convert_LPTSTR);

	     return this->svt_tmp ;
	};

/**
 * @fn ::wstring String_To_Wstring(::std::string convert_string) 
 *
 * @brief Funkcia konvertovania "STRING" -> "WSTRING" (wide string).
 *
 * @param convert_string Vstupná hodnota dátového typu "STRING".
 *
 * @return Funkcia vracia vstupnú hodnotu pretypovanú na "WSTRING".
 */
    ::wstring String_To_Wstring(::std::string convert_string) 
	{
         ::wstring _0convert(convert_string.length(), L' ');
         ::copy(convert_string.begin(), convert_string.end(), _0convert.begin());

         return _0convert;
	};

/**
 * @fn ::LPVOID String_To_Lpvoid(::std::string convert_string) 
 *
 * @brief Funkcia konvertovania "STRING" -> "LPVOID" (pointer to a void object).
 *
 * @param convert_string Vstupná hodnota dátového typu "STRING".
 *
 * @return Funkcia vracia vstupnú hodnotu pretypovanú na "LPVOID".
 */
	::LPVOID String_To_Lpvoid(::std::string convert_string)
	{
		 return (::LPVOID) convert_string.c_str();
	};

/**
 * @fn ::std::string Wstring_To_String (const ::wstring & convert_wstring) 
 *
 * @brief Funkcia konvertovania "WSTRING" (Wide string) -> "STRING".
 *
 * @param convert_string Vstupná hodnota dátového typu "WSTRING".
 *
 * @return Funkcia vracia vstupnú hodnotu pretypovanú na "STRING".
 */
	::std::string Wstring_To_String (const ::wstring & convert_wstring)
	{
         this->cnvt_tmp = (int)convert_wstring.length() + 1;
         this->cnvt_l = ::WideCharToMultiByte(CP_ACP, 0, convert_wstring.c_str(), this->cnvt_tmp, 0, 0, 0, 0);
         this->wsbuffer = new char[this->cnvt_l];
         ::WideCharToMultiByte(CP_ACP, 0, convert_wstring.c_str(), this->cnvt_tmp, this->wsbuffer, this->cnvt_l, 0, 0);
         ::string ret_val(this->wsbuffer);
         delete [] this->wsbuffer;

         return ret_val;
	};

/**
 * @fn int String_To_Int (::std::string convert_string)
 *
 * @brief Funkcia konvertovania "STRING" -> "INTEGER".
 *
 * @param convert_string Vstupná hodnota dátového typu "STRING".
 *
 * @return Funkcia vracia vstupnú hodnotu pretypovanú na "INTEGER".
 */
	int String_To_Int (::std::string convert_string)
	{
	     this->cnvt_tmp = ::atoi(convert_string.c_str());

	     return this->cnvt_tmp;
	};

/**
 * @fn ::std::string Int_To_String (int convert_integer)
 *
 * @brief Funkcia konvertovania "INTEGER" -> "STRING".
 *
 * @param convert_string Vstupná hodnota dátového typu "INTEGER".
 *
 * @return Funkcia vracia vstupnú hodnotu pretypovanú na "STRING".
 */
	::std::string Int_To_String (int convert_integer)
	{  
		this->dstreambuffer << convert_integer;
		this->svt_tmp = this->dstreambuffer.str();
		this->ClearBuffer();

	    return this->svt_tmp;
	};

/**
 * @fn ::std::string Float_To_String (float convert_float)
 *
 * @brief Funkcia konvertovania "FLOAT" -> "STRING".
 *
 * @param convert_string Vstupná hodnota dátového typu "FLOAT".
 *
 * @return Funkcia vracia vstupnú hodnotu pretypovanú na "STRING".
 */
	::std::string Float_To_String (float convert_float)
	{ 
		this->dstreambuffer << convert_float;
        this->svt_tmp = this->dstreambuffer.str();
        this->ClearBuffer();

	    return this->svt_tmp;
	};

/**
 * @fn float String_To_Float (::std::string convert_string)
 *
 * @brief Funkcia konvertovania "STRING" -> "FLOAT".
 *
 * @param convert_string Vstupná hodnota dátového typu "STRING".
 *
 * @return Funkcia vracia vstupnú hodnotu pretypovanú na "FLOAT".
 */
	float String_To_Float (::std::string convert_string)
	{
        stringstream _du_(convert_string);
         _du_ >> this->dbuffer;
         return this->dbuffer;
	};

/**
 * @fn ::std::string Char_To_String (char convert_char)
 *
 * @brief Funkcia konvertovania "CHAR" -> "STRING".
 *
 * @param convert_string Vstupná hodnota dátového typu "CHAR".
 *
 * @return Funkcia vracia vstupnú hodnotu pretypovanú na "STRING".
 */
	::std::string Char_To_String (char convert_char)
	{
	    this->dstreambuffer << convert_char;
        this->dstreambuffer >> this->svt_tmp;
		this->ClearBuffer();

		return this->svt_tmp; 
	};

/**
 * @fn int Char_To_Int (char convert_char)
 *
 * @brief Funkcia konvertovania "CHAR" -> "INTEGER".
 *
 * @param convert_string Vstupná hodnota dátového typu "CHAR".
 *
 * @return Funkcia vracia vstupnú hodnotu pretypovanú na "INTEGER".
 */
	int Char_To_Int (char convert_char)
	{
		this->cnvt_tmp = ::std::atoi(&convert_char);

		return this->cnvt_tmp;
	};

    ~Diall_Convert (void)
	{
         delete [] this->convertstring;
	};

	private:
		void ClearBuffer(void)
		{
			this->dstreambuffer.str(::std::string());
			this->dstreambuffer.clear();
		}

};
 
//nastavenie nulovej instancie pre pristup Singletonu//
Diall_Convert * ::Diall_Convert::instance = nullptr;

